@@ -122,6 +122,14 @@ toplevelstatement(stats_t *stats)
 	    }
 	    break;
 
+	case T_NAMESPACE:
+	    if (maybenamespace())
+	    {
+		/* no trailing semicolon for namespaces */
+		endofstatement = TRUE;
+	    }
+	    break;
+
 	case '(':
 	    /* possible start of function */
 	    functionDefLine = Line;
@@ -235,6 +243,67 @@ maybeclass()
     return isclass;
 }
 
+int
+maybenamespace()
+/*
+ * We've just seen "namespace" at the top level in a file so
+ * we may be entering a definition of same (if we next find "token {").
+ * Return 1 if this is a
+ * namespace definition else restore the name and return 0.
+ */
+{
+    char nsname[256], dummy[256];
+    int isns = 0;
+    int c;
+
+    if ((c = gettoken(nsname, NULL, NULL)) == T_IDENT)
+    {
+	/* "namespace name" */
+	switch(c = gettoken(dummy, NULL, NULL))
+	{
+	case '{':	/* "namespace name {" */
+	    break;
+
+	default:
+	    /* if we fail to get "namespace name {" */
+	    ungettoken(c, dummy);
+	}
+    }
+    else if (c == '{')	/* Unnamed namespace */
+    {
+	/* "namespace {" */
+	strcpy(nsname, "anonymous_namespace");
+    }
+    else
+    {
+	/* "namespace BOGUS" -- is C code using a variable "namespace"? */
+	ungettoken(c, dummy);
+    }
+
+    if (isns = (c == '{'))
+    {
+	stats_t *ns = stats_push(nsname, STATS_NAMESPACE);
+
+	while ((c = gettoken(dummy, NULL, NULL)) != '}')
+	{
+	    if (c == EOF)
+	    {
+		fileerror("unexpected EOF");
+		break;
+	    }
+	    else
+		ungettoken(c, dummy);
+
+	    toplevelstatement(ns);
+	}
+
+	stats_pop(ns);
+    }
+
+    return isns;
+}
+
+
 void
 findsemicolon()
 {
